#include "defines.h"
#include <fcntl.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <linux/soundcard.h>
#include <libintl.h>
#define _(String) gettext (String)

extern double io_sound_count;
extern int io_sound_age;
extern flag_t nflag, fullspeed;
unsigned io_sound_val = 0;
flag_t io_stop_happened = 0;
flag_t telegraph_enabled = 0; 	/* Default */

io_init() {
	sound_init();
	tape_init();
	return OK;
}

io_read(addr, word)
c_addr addr;
d_word *word;
{
	int tape_bit;
	tape_bit = tape_read() << 5;

	/* the high byte is 0200 for BK-0010
	 * and 0300 for BK-0011
	 */
	*word = 0100000 | (bkmodel << 14) |
		(telegraph_enabled ? serial_read() : 0200) |
		key_pressed |
		tape_bit |
		io_stop_happened;
	io_stop_happened = 0;
	return OK;
}

/* Include tape drive relay into sound as well */
io_write(addr, word)
c_addr addr;
d_word word;
{
	d_word offset = addr - IO_REG;
	unsigned oldval = io_sound_val;
	if (bkmodel && word & 04000) {
		pagereg_write(word);
		return OK;
	}
	io_sound_val = word & 0300;
	if (io_sound_val != oldval) {
		if (fullspeed) io_sound_count = ticks;
		    io_sound_age = 0;
	}
	/* status, value */
	tape_write((word >> 7) & 1, (word >> 6) & 1);
	if (telegraph_enabled) {
		serial_write(word);
	}
	return OK;
}

io_bwrite(c_addr addr, d_byte byte) {
	d_word offset = addr - IO_REG;
	unsigned oldval = io_sound_val;
	if (offset == 0) {
	    io_sound_val = byte & 0300;
	    if (io_sound_val != oldval) {
		    if (fullspeed) io_sound_count = ticks;
		    io_sound_age = 0;
	    }
	    tape_write((byte >> 7) & 1, (byte >> 6) & 1);
	    if (telegraph_enabled) {
		    serial_write(byte);
	    }
	} else if (bkmodel && byte & 010) {
		pagereg_write(byte << 8);
	}
	return OK;
}

line_init() {
}

line_read(addr, word)
c_addr addr;
d_word *word;
{
	fprintf(stderr, _("Reading %06o\n"), addr);
	*word = 0;
	return OK;
}

line_write(addr, word)
c_addr addr;
d_word word;
{
	/* fprintf(stderr, "Writing %06o to %06o\n", word, addr); */
	return OK;
}

line_bwrite(addr, byte)
c_addr addr;
d_byte byte;
{
	fprintf(stderr, _("Writing %03o to %06o\n"), byte, addr);
	return OK;
}
